#include <stdio.h>
#include <memory>

/** Base class */
class Base {
public:
  /** Public, non-virtual interface. Interally despatches virtual_hook() */
  void exec() { virtual_hook(); }
private:
  /** A function despatched via virtual-function lookup */
  virtual void virtual_hook() { }
};

/** A class derived from Base */
class Derived : public Base {
private:
  // The documentation for this should be inherited
  virtual void virtual_hook() { printf("Derived hook\n"); }
};

int main(int argc, char *argv[])
{
  std::auto_ptr<Base> b(new Derived);
  b->exec();

  return 0;
}
